@@ -41,6 +41,11 @@ const Ci = Components.interfaces;
 Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
 
 let modules = {
+  // about:blank has some bad loading behavior we can avoid, if we use an alias
+  empty: {
+    uri: "about:blank",
+    privileged: false
+  },
   fennec: {
     uri: "chrome://browser/content/about.xhtml",
     privileged: true
@@ -49,18 +54,22 @@ let modules = {
   get firefox() this.fennec,
 
   firstrun: {
-    uri: "chrome://browser/content/firstrun.xhtml",
+    uri: "chrome://browser/content/firstrun/firstrun.xhtml",
     privileged: true
   },
   rights: {
-//@line 59 "/builds/slave/linux-fennec-electrolysis-nightly/build/electrolysis/mobile/components/AboutRedirector.js"
+//@line 64 "/builds/slave/2.1-mob-lnx-ntly/build/mozilla-2.1/mobile/components/AboutRedirector.js"
     uri: "chrome://global/content/aboutRights-unbranded.xhtml",
-//@line 61 "/builds/slave/linux-fennec-electrolysis-nightly/build/electrolysis/mobile/components/AboutRedirector.js"
+//@line 66 "/builds/slave/2.1-mob-lnx-ntly/build/mozilla-2.1/mobile/components/AboutRedirector.js"
     privileged: false
   },
   certerror: {
     uri: "chrome://browser/content/aboutCertError.xhtml",
     privileged: true
+  },
+  home: {
+    uri: "chrome://browser/content/aboutHome.xhtml",
+    privileged: true
   }
 }
 
@@ -99,46 +108,48 @@ AboutGeneric.prototype = {
   }
 };
 
+function AboutEmpty() {}
+AboutEmpty.prototype = {
+  __proto__: AboutGeneric.prototype,
+  classID: Components.ID("{433d2d75-5923-49b0-854d-f37267b03dc7}")
+}
+
 function AboutFirstrun() {}
 AboutFirstrun.prototype = {
   __proto__: AboutGeneric.prototype,
-  classDescription: "About Firstrun",
-  contractID: "@mozilla.org/network/protocol/about;1?what=firstrun",
   classID: Components.ID("{077ea23e-0f22-4168-a744-8e444b560197}")
 }
 
 function AboutFennec() {}
 AboutFennec.prototype = {
   __proto__: AboutGeneric.prototype,
-  classDescription: "About Fennec",
-  contractID: "@mozilla.org/network/protocol/about;1?what=fennec",
   classID: Components.ID("{842a6d11-b369-4610-ba66-c3b5217e82be}")
 }
 
 function AboutFirefox() {}
 AboutFirefox.prototype = {
   __proto__: AboutGeneric.prototype,
-  classDescription: "About Firefox",
-  contractID: "@mozilla.org/network/protocol/about;1?what=firefox",
   classID: Components.ID("{dd40c467-d206-4f22-9215-8fcc74c74e38}")  
 }
 
 function AboutRights() {}
 AboutRights.prototype = {
   __proto__: AboutGeneric.prototype,
-  classDescription: "About Rights",
-  contractID: "@mozilla.org/network/protocol/about;1?what=rights",
   classID: Components.ID("{3b988fbf-ec97-4e1c-a5e4-573d999edc9c}")
 }
 
 function AboutCertError() {}
 AboutCertError.prototype = {
   __proto__: AboutGeneric.prototype,
-  classDescription: "About Certificate Error",
-  contractID: "@mozilla.org/network/protocol/about;1?what=certerror",
   classID: Components.ID("{972efe64-8ac0-4e91-bdb0-22835d987815}")
 }
 
-function NSGetModule(compMgr, fileSpec)
-  XPCOMUtils.generateModule([AboutFirstrun, AboutFennec, AboutRights,
-                             AboutCertError, AboutFirefox]);
+function AboutHome() {}
+AboutHome.prototype = {
+  __proto__: AboutGeneric.prototype,
+  classID: Components.ID("{b071364f-ab68-4669-a9db-33fca168271a}")
+}
+
+const components = [AboutEmpty, AboutFirstrun, AboutFennec, AboutRights,
+                    AboutCertError, AboutFirefox, AboutHome];
+const NSGetFactory = XPCOMUtils.generateNSGetFactory(components);
