@@ -1,27 +1,38 @@
-//@line 44 "/builds/slave/linux-fennec-lorentz-nightly/build/firefox-lorentz/mobile/components/Sidebar.js"
+//@line 43 "/builds/slave/2.1-mob-lnx-ntly/build/mozilla-2.1/mobile/components/Sidebar.js"
 
-const Cc = Components.classes;
 const Ci = Components.interfaces;
-const Cr = Components.results;
+const Cc = Components.classes;
 const Cu = Components.utils;
 
 Cu.import("resource://gre/modules/XPCOMUtils.jsm");
-
-const JAVASCRIPT_GLOBAL_PROPERTY_CATEGORY = "JavaScript global property";
-
-var gStrBundleService = null;
-function srGetStrBundle(path) {
-  if (!gStrBundleService) {
-    gStrBundleService = Cc["@mozilla.org/intl/stringbundle;1"].
-                        getService(Ci.nsIStringBundleService);
-  }
-
-  return gStrBundleService.createBundle(path);
-}
+Cu.import("resource://gre/modules/Services.jsm");
 
 function Sidebar() {
-  this.searchService = Cc["@mozilla.org/browser/search-service;1"].
-                       getService(Ci.nsIBrowserSearchService);
+  // Depending on if we are in the parent or child, prepare to remote
+  // certain calls
+  var appInfo = Cc["@mozilla.org/xre/app-info;1"];
+  if (!appInfo || appInfo.getService(Ci.nsIXULRuntime).processType == Ci.nsIXULRuntime.PROCESS_TYPE_DEFAULT) {
+    // Parent process
+
+    this.inContentProcess = false;
+
+    // Used for wakeups service. FIXME: clean up with bug 593407
+    this.wrappedJSObject = this;
+
+    // Setup listener for child messages. We don't need to call
+    // addMessageListener as the wakeup service will do that for us.
+    this.receiveMessage = function(aMessage) {
+      switch (aMessage.name) {
+        case "Sidebar:AddSearchProvider":
+          this.AddSearchProvider(aMessage.json.descriptionURL);
+      }
+    };
+  } else {
+    // Child process
+
+    this.inContentProcess = true;
+    this.messageManager = Cc["@mozilla.org/childprocessmessagemanager;1"].getService(Ci.nsISyncMessageSender);
+  }
 }
 
 Sidebar.prototype = {
@@ -40,15 +51,13 @@ Sidebar.prototype = {
     } catch(ex) {
       Cu.reportError("Invalid argument passed to window.sidebar.addSearchEngine: " + ex);
       
-      var searchBundle = srGetStrBundle("chrome://global/locale/search/search.properties");
-      var brandBundle = srGetStrBundle("chrome://branding/locale/brand.properties");
+      var searchBundle = Services.strings.createBundle("chrome://global/locale/search/search.properties");
+      var brandBundle = Services.strings.createBundle("chrome://branding/locale/brand.properties");
       var brandName = brandBundle.GetStringFromName("brandShortName");
       var title = searchBundle.GetStringFromName("error_invalid_engine_title");
       var msg = searchBundle.formatStringFromName("error_invalid_engine_msg",
                                                   [brandName], 1);
-      var promptService = Cc["@mozilla.org/embedcomp/prompt-service;1"].
-                          getService(Ci.nsIPromptService);
-      promptService.alert(null, title, msg);
+      Services.prompt.alert(null, title, msg);
       return false;
     }
     
@@ -87,7 +96,7 @@ Sidebar.prototype = {
     else
       dataType = Ci.nsISearchEngine.DATA_XML;
 
-    this.searchService.addEngine(engineURL, dataType, iconURL, true);
+    Services.search.addEngine(engineURL, dataType, iconURL, true);
   },
 
   // =========================== nsISidebarExternal ===========================
@@ -97,9 +106,15 @@ Sidebar.prototype = {
   AddSearchProvider: function AddSearchProvider(aDescriptionURL) {
     if (!this._validateSearchEngine(aDescriptionURL, ""))
       return;
-  
+
+    if (this.inContentProcess) {
+      this.messageManager.sendAsyncMessage("Sidebar:AddSearchProvider",
+        { descriptionURL: aDescriptionURL });
+      return;
+    }
+
     const typeXML = Ci.nsISearchEngine.DATA_XML;
-    this.searchService.addEngine(aDescriptionURL, typeXML, "", true);
+    Services.search.addEngine(aDescriptionURL, typeXML, "", true);
   },
 
   // This function exists to implement window.external.IsSearchProviderInstalled(),
@@ -133,14 +148,6 @@ Sidebar.prototype = {
 
   // XPCOMUtils stuff
   classID: Components.ID("{22117140-9c6e-11d3-aaf1-00805f8a4905}"),
-  className: "Sidebar JS Component",
-  contractID: "@mozilla.org/sidebar;1",
-  _xpcom_categories: [{ category: JAVASCRIPT_GLOBAL_PROPERTY_CATEGORY,
-                        entry: "sidebar"},
-                      { category: JAVASCRIPT_GLOBAL_PROPERTY_CATEGORY,
-                        entry: "external"}]
-}
+};
 
-function NSGetModule(compMgr, fileSpec) {
-    return XPCOMUtils.generateModule([Sidebar]);
-}
+const NSGetFactory = XPCOMUtils.generateNSGetFactory([Sidebar]);
