@@ -1,4 +1,4 @@
-//@line 40 "/builds/slave/linux-fennec-lorentz-nightly/build/firefox-lorentz/toolkit/components/viewsource/content/viewPartialSource.js"
+//@line 40 "/builds/slave/2.1-mob-lnx-ntly/build/mozilla-2.1/toolkit/components/viewsource/content/viewPartialSource.js"
 
 var gDebug = 0;
 var gLineCount = 0;
@@ -8,7 +8,7 @@ var gTargetNode = null;
 
 var gEntityConverter = null;
 var gWrapLongLines = false;
-const gViewSourceCSS = 'resource://gre/res/viewsource.css';
+const gViewSourceCSS = 'resource://gre-resources/viewsource.css';
 const NS_XHTML = 'http://www.w3.org/1999/xhtml';
 
 // These are markers used to delimit the selection during processing. They
@@ -43,6 +43,11 @@ function onLoadViewPartialSource()
   else
     viewPartialSourceForFragment(window.arguments[2], window.arguments[3]);
 
+  gBrowser.droppedLinkHandler = function (event, url, name) {
+    viewSource(url)
+    event.preventDefault();
+  }
+
   window.content.focus();
 }
 
@@ -163,9 +168,9 @@ function viewPartialSourceForSelection(selection)
 
   // all our content is held by the data:URI and URIs are internally stored as utf-8 (see nsIURI.idl)
   var loadFlags = Components.interfaces.nsIWebNavigation.LOAD_FLAGS_NONE;
-  getBrowser().webNavigation
-              .loadURI("view-source:data:text/html;charset=utf-8," + encodeURIComponent(tmpNode.innerHTML),
-                       loadFlags, null, null, null);
+  getWebNavigation().loadURI("view-source:data:text/html;charset=utf-8,"
+                             + encodeURIComponent(tmpNode.innerHTML),
+                             loadFlags, null, null, null);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -199,8 +204,8 @@ function getPath(ancestor, node)
 // on the inflated view-source DOM
 function drawSelection()
 {
-  getBrowser().contentDocument.title =
-    getViewSourceBundle().getString("viewSelectionSourceTitle");
+  gBrowser.contentDocument.title =
+    gViewSourceBundle.getString("viewSelectionSourceTitle");
 
   // find the special selection markers that we added earlier, and
   // draw the selection between the two...
@@ -222,7 +227,7 @@ function drawSelection()
   var replaceString = findService.replaceString;
 
   // setup our find instance
-  var findInst = getBrowser().webBrowserFind;
+  var findInst = gBrowser.webBrowserFind;
   findInst.matchCase = true;
   findInst.entireWord = false;
   findInst.wrapFind = true;
@@ -233,8 +238,10 @@ function drawSelection()
   var startLength = MARK_SELECTION_START.length;
   findInst.findNext();
 
-  var contentWindow = getBrowser().contentDocument.defaultView;
-  var selection = contentWindow.getSelection();
+  var selection = content.getSelection();
+  if (!selection.rangeCount)
+    return;
+
   var range = selection.getRangeAt(0);
 
   var startContainer = range.startContainer;
@@ -264,13 +271,10 @@ function drawSelection()
   // the selection, whereas in this situation, it is more user-friendly
   // to scroll at the beginning. So we override the default behavior here
   try {
-    getBrowser().docShell
-                .QueryInterface(Components.interfaces.nsIInterfaceRequestor)
-                .getInterface(Components.interfaces.nsISelectionDisplay)
-                .QueryInterface(Components.interfaces.nsISelectionController)
-                .scrollSelectionIntoView(Components.interfaces.nsISelectionController.SELECTION_NORMAL,
-                                         Components.interfaces.nsISelectionController.SELECTION_ANCHOR_REGION,
-                                         true);
+    getSelectionController().scrollSelectionIntoView(
+                               Ci.nsISelectionController.SELECTION_NORMAL,
+                               Ci.nsISelectionController.SELECTION_ANCHOR_REGION,
+                               true);
   }
   catch(e) { }
 
@@ -311,7 +315,7 @@ function viewPartialSourceForFragment(node, context)
     return;
 
   // serialize
-  var title = getViewSourceBundle().getString("viewMathMLSourceTitle");
+  var title = gViewSourceBundle.getString("viewMathMLSourceTitle");
   var wrapClass = gWrapLongLines ? ' class="wrap"' : '';
   var source =
     '<!DOCTYPE html>'
@@ -330,7 +334,7 @@ function viewPartialSourceForFragment(node, context)
   ; // end
 
   // display
-  getBrowser().loadURI("data:text/html;charset=utf-8," + encodeURIComponent(source));
+  gBrowser.loadURI("data:text/html;charset=utf-8," + encodeURIComponent(source));
 }
 
 ////////////////////////////////////////////////////////////////////////////////
