@@ -1,4 +1,4 @@
-//@line 40 "/builds/slave/linux-fennec-electrolysis-nightly/build/electrolysis/toolkit/components/viewsource/content/viewSource.js"
+//@line 40 "/builds/slave/2.1-mob-lnx-ntly/build/mozilla-2.1/toolkit/components/viewsource/content/viewSource.js"
 
 const Cc = Components.classes;
 const Ci = Components.interfaces;
@@ -55,7 +55,11 @@ function onLoadViewSource()
 {
   viewSource(window.arguments[0]);
   document.commandDispatcher.focusedWindow = content;
-      
+  gBrowser.droppedLinkHandler = function (event, url, name) {
+    viewSource(url)
+    event.preventDefault();
+  }
+
   if (!isHistoryEnabled()) {
     // Disable the BACK and FORWARD commands and hide the related menu items.
     var viewSourceNavigation = document.getElementById("viewSourceNavigation");
@@ -333,38 +337,40 @@ function ViewSourceReload()
                            Ci.nsIWebNavigation.LOAD_FLAGS_BYPASS_CACHE);
 }
 
-// Strips the |view-source:| for editPage()
-function ViewSourceEditPage()
-{
-  editPage(window.content.location.href.substring(12), window, false);
-}
-
 // Strips the |view-source:| for saveURL()
 function ViewSourceSavePage()
 {
   saveURL(window.content.location.href.substring(12), null, "SaveLinkTitle");
 }
 
-function onEnterPP()
-{
-  var toolbox = document.getElementById("viewSource-toolbox");
-  toolbox.hidden = true;
-}
-
-function onExitPP()
-{
-  var toolbox = document.getElementById("viewSource-toolbox");
-  toolbox.hidden = false;
-}
-
-function getPPBrowser() {
-  return gBrowser;
-}
-
-// printUtils.js uses this
-function getNavToolbox()
-{
-  return document.getElementById("appcontent");
+var PrintPreviewListener = {
+  getPrintPreviewBrowser: function () {
+    var browser = document.getElementById("ppBrowser");
+    if (!browser) {
+      browser = document.createElement("browser");
+      browser.setAttribute("id", "ppBrowser");
+      browser.setAttribute("flex", "1");
+      document.getElementById("appcontent").
+        insertBefore(browser, document.getElementById("FindToolbar"));
+    }
+    return browser;
+  },
+  getSourceBrowser: function () {
+    return gBrowser;
+  },
+  getNavToolbox: function () {
+    return document.getElementById("appcontent");
+  },
+  onEnter: function () {
+    var toolbox = document.getElementById("viewSource-toolbox");
+    toolbox.hidden = true;
+    gBrowser.collapsed = true;
+  },
+  onExit: function () {
+    document.getElementById("ppBrowser").collapsed = true;
+    gBrowser.collapsed = false;
+    document.getElementById("viewSource-toolbox").hidden = false;
+  }
 }
 
 function getWebNavigation()
