@@ -1,4 +1,4 @@
-//@line 45 "/builds/slave/linux-fennec-electrolysis-nightly/build/electrolysis/toolkit/mozapps/downloads/content/downloads.js"
+//@line 45 "/builds/slave/2.1-mob-lnx-ntly/build/mozilla-2.1/toolkit/mozapps/downloads/content/downloads.js"
 
 ////////////////////////////////////////////////////////////////////////////////
 //// Globals
@@ -20,6 +20,9 @@ Cu.import("resource://gre/modules/PluralForm.jsm");
 const nsIDM = Ci.nsIDownloadManager;
 
 let gDownloadManager = Cc["@mozilla.org/download-manager;1"].getService(nsIDM);
+let gDownloadManagerUI = Cc["@mozilla.org/download-manager-ui;1"].
+                         getService(Ci.nsIDownloadManagerUI);
+
 let gDownloadListener = null;
 let gDownloadsView = null;
 let gSearchBox = null;
@@ -124,6 +127,8 @@ function downloadCompleted(aDownload)
 
     if (gDownloadManager.activeDownloadCount == 0)
       document.title = document.documentElement.getAttribute("statictitle");
+
+    gDownloadManagerUI.getAttention();
   }
   catch (e) { }
 }
@@ -246,7 +251,7 @@ function openDownload(aDownload)
       dontAsk = !pref.getBoolPref(PREF_BDM_ALERTONEXEOPEN);
     } catch (e) { }
 
-//@line 306 "/builds/slave/linux-fennec-electrolysis-nightly/build/electrolysis/toolkit/mozapps/downloads/content/downloads.js"
+//@line 311 "/builds/slave/2.1-mob-lnx-ntly/build/mozilla-2.1/toolkit/mozapps/downloads/content/downloads.js"
 
     if (!dontAsk) {
       var strings = document.getElementById("downloadStrings");
@@ -419,6 +424,7 @@ function Startup()
             getService(Ci.nsIObserverService);
   obs.addObserver(gDownloadObserver, "download-manager-remove-download", false);
   obs.addObserver(gDownloadObserver, "private-browsing", false);
+  obs.addObserver(gDownloadObserver, "browser-lastwindow-close-granted", false);
 
   // Clear the search box and move focus to the list on escape from the box
   gSearchBox.addEventListener("keypress", function(e) {
@@ -428,6 +434,8 @@ function Startup()
       e.preventDefault();
     }
   }, false);
+
+//@line 501 "/builds/slave/2.1-mob-lnx-ntly/build/mozilla-2.1/toolkit/mozapps/downloads/content/downloads.js"
 }
 
 function Shutdown()
@@ -438,6 +446,7 @@ function Shutdown()
             getService(Ci.nsIObserverService);
   obs.removeObserver(gDownloadObserver, "private-browsing");
   obs.removeObserver(gDownloadObserver, "download-manager-remove-download");
+  obs.removeObserver(gDownloadObserver, "browser-lastwindow-close-granted");
 
   clearTimeout(gBuilder);
   gStmt.reset();
@@ -481,6 +490,13 @@ let gDownloadObserver = {
           }, 0);
         }
         break;
+      case "browser-lastwindow-close-granted":
+//@line 557 "/builds/slave/2.1-mob-lnx-ntly/build/mozilla-2.1/toolkit/mozapps/downloads/content/downloads.js"
+        if (gDownloadManager.activeDownloadCount == 0) {
+          setTimeout(gCloseDownloadManager, 0);
+        }
+//@line 561 "/builds/slave/2.1-mob-lnx-ntly/build/mozilla-2.1/toolkit/mozapps/downloads/content/downloads.js"
+        break;
     }
   }
 };
@@ -626,12 +642,25 @@ function buildContextMenu(aEvent)
 
   return false;
 }
-
 ////////////////////////////////////////////////////////////////////////////////
 //// Drag and Drop
-
 var gDownloadDNDObserver =
 {
+  onDragStart: function (aEvent)
+  {
+    if (!gDownloadsView.selectedItem)
+      return;
+    var dl = gDownloadsView.selectedItem;
+    var f = getLocalFileFromNativePathOrUrl(dl.getAttribute("file"));
+    if (!f.exists())
+      return;
+
+    var dt = aEvent.dataTransfer;
+    dt.mozSetDataAt("application/x-moz-file", f, 0);
+    dt.effectAllowed = "copyMove";
+    dt.addElement(dl);
+  },
+
   onDragOver: function (aEvent)
   {
     var types = aEvent.dataTransfer.types;
